//=============================================================================
// BotzWP.
// INSTRUCCIONES:
//	=Sentidos
//		- Especificar siempre ambos nodes y ambas junciones
//		- El orden es irrelevante en los nodes
//		- Nunca poner un goal como juncion[0], no unir dos goals por un camino
//	=Precauciones
//		- No dejar caminos rotos, se sale el juego si pasa. (Assert == x)
//		- Se recomienda terminar todo antes de usar.
//		- SIEMPRE especificar ambas junciones, y no deben ser la misma.
//		- Si el camino es un punto final, agregar un BotzGoal, que simula una juncion
//	=Consejos
//		- Puedes probar los WP antes de terminarlos, siempre que no pongas un botz
//		- Hazte una lista de las junciones y goals en un papel, y sus descripciones
//			As no omitirs ninguno en BestPaths[]
//
// For testing purposes, do not modify or redistribute
// Have suggestions? Contact me.
// caco_fff@hotmail.com - Higor at UT99.org
//=============================================================================
class BotzWP expands BaseBotzWP;

#exec TEXTURE IMPORT NAME=BWP_Normal FILE=..\CompileData\BWP_Normal.bmp FLAGS=2

var float FirstTick;
var bool bCanNow;
var BotzPathInfo PathInfo;
var Mover MyLift;
var(SimpleNav) BaseBotzWP Nodes[2];
var(SimpleNav) BotzJunction Junctions[2];
var(SimpleNav) bool NoLink; //Crear one-way-path, destino tiene el WP al lado de la juncion NoLink
var(SimpleNav) teleporter TeleP;

var() Name MustConnect;

var() name NeverConnectName;
var BaseBotzWp NeverConnect;

function InitializeWp()
{
	local int i;
	local BotzJunction TempJ;

	FirstTick = Level.TimeSeconds;

	For (i=0 ; i<2 ; i++ )
	{
		Assert( Junctions[i] != none);
		Assert( Nodes[i] != none);
		TempJ = BotzJunction(Nodes[i]);
		if (TempJ != none)
			LinkJunc(TempJ);
	}
}

function LinkJunc( BotzJunction Junc)
{
	local int i;
	local BotzJunction TargetJunc;

	For (i=0 ; i<2 ; i++)
	{
		if ( Junctions[i] == Junc )
			TargetJunc = Junctions[ abs(i - 1)];
		else
			TargetJunc = Junctions[i];
	}

	For ( i=0 ; i<8 ; i++ )
		if (Junc.LinkedJunctions[i] == none)
			break;

	if ( Junc.IsA('BotzGoal') )
	{
		BotzGoal(Junc).TheJunc = TargetJunc;
		BotzGoal(Junc).TheNode = self;
		Junc.LinkedJunctions[0] = TargetJunc;
		Junc.LinkedPaths[0] = self;
		return;
	}

	Junc.LinkedJunctions[i] = TargetJunc;
	Junc.LinkedPaths[i] = self;
}

function BaseBotzWP SearchAPath(BaseBotzWP PathTarget, BaseBotzWP Origen, out float Dist, optional bool bIgnoreJunc)
{
	local BotzJunction Closest[2];
	local float Distance[2];
	local float BestDist;
	local int BestI;
	local float OtherDist, DaDist;
	local BaseBotzWp tempResult;

	if ( PathInfo.PathInWay(PathTarget) )
	{
		Dist = PathInfo.GetDist(PathTarget, self);
		return PathInfo.FindNext( self, PathTarget);
	}


	FindJunctionsTo( self, Closest, Distance);
	Closest[0].SearchAPath( PathTarget, Closest[0], DaDist);
	BestDist = Distance[0] + DaDist;
	BestI = 0;

	if ( Closest[1] != none )
	{
		Closest[1].SearchAPath( PathTarget, Closest[1], DaDist);
		OtherDist = DaDist + Distance[1];
		if ( OtherDist < BestDist )
		{
			BestDist = OtherDist;
			BestI = 1;
		}
	}
	Dist = BestDist;
	tempResult = PathInfo.FindNext( self, Closest[BestI] );
	if ( (tempResult != PathTarget) && (BotzJunction(tempResult) != none) && (bIgnoreJunc) )
		return tempResult.SearchAPath( PathTarget, self, Dist, false);
	return tempResult;
}

function StartPathInfo()
{
	local int i, j;

	if ( PathInfo != none )
		Return;

	PathInfo = Spawn(class'BotzPathInfo',,, vect(0,0,0), rot(0,0,0) );
	PathInfo.BeginPathing( self);

/*	if (NoLink)
		For ( i=0 ; i<2 ; i++ )
			if ( Nodes[i].IsA('BotzJunction') )
				For ( j=0 ; j<8 ; j++ )
					if ( BotzJunction(Nodes[i]).LinkedPaths[j] == self )
					{
						BotzJunction(Nodes[i]).LinkedPaths[j] = none;
						BotzJunction(Nodes[i]).LinkedJunctions[j] = none;
						return;
					}*/
}

defaultproperties
{
     AddedOffset=20.000000
     Texture=Texture'FerBotz.BWP_Normal'
     CollisionHeight=40.000000
     bCollideActors=True
}
